#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "config.h"
#include "imapfilter.h"
#include "file.h"
#include "connect.h"
#include "imap.h"
#include "log.h"


int options;			/* Program options. */
extern int sock;
extern account_data account;


/*
 * In the begining there was main()...
 */
int main(int argc, char **argv)
{
    char *config = NULL;	/* Configuration file. */
    int c;

    options = OPTION_DETAILS_NORMAL;

    prepare_account();

    while ((c = getopt(argc, argv, "c:hl:qstv")) != -1) {
	switch (c) {
	case 'c':
	    config = optarg;
	    break;
	case 'h':
	    usage();
	    exit(1);
	    break;
	case 'l':
	    strncpy(account.logfile, optarg, FILENAME_MAX);
	    break;
	case 'q':
	    options &= OPTION_DETAILS_CLEAR;
	    options |= OPTION_DETAILS_QUITE;
	    break;
	case 's':
	    options |= OPTION_SHOW_HEADERS;
	    break;
	case 't':
	    options |= OPTION_TEST_MODE;
	    break;
	case 'v':
	    options &= OPTION_DETAILS_CLEAR;
	    options |= OPTION_DETAILS_VERBOSE;
	    break;
	default:
	    usage();
	    exit(1);
	    break;
	}
    }

    if (read_config(config) || open_logfile())
	exit(2);

    if (establish_connection()) {
	close_logfile();
	exit(3);
    }

    clear_stream();

#ifdef DEBUG
    imap_noop();
#endif

    imap_login();
    imap_select();

    apply_filters();

    imap_logout();

    close_connection();

    close_logfile();

    exit(0);
}


/*
 * Search and if found (and not in test mode) delete any messages
 * that match the defined filters.
 */
void apply_filters(void)
{
    char results[SEARCH_RESULTS_MAX];

    if (!imap_search(results) && results[0]) {
	delete_messages(results);
	if (!(options & OPTION_TEST_MODE))
	    imap_expunge();
    }
}


/*
 * Prints a very brief usage message.
 */
void usage(void)
{
    fprintf(stderr,
	    "usage: imapfilter [-hqstv] [-c configfile] [-l logfile]\n");
}
